[−][src]Struct cached_path::Meta
Holds information about a cached resource.
Fields
resource: StringThe original resource name.
resource_path: PathBufPath to the cached resource.
meta_path: PathBufPath to the serialized meta.
etag: Option<String>The ETAG of the resource from the time it was cached, if there was one.
expires: Option<f64>Time that the freshness of this cached resource will expire.
creation_time: f64Time this version of the resource was cached.
Methods
impl Meta[src]
pub async fn from_cache<'_>(resource_path: &'_ Path) -> Result<Self, Error>[src]
Get the Meta from a cached resource.
Examples
use cached_path::{cached_path, Meta}; let resource = "https://github.com/epwalsh/rust-cached-path/blob/master/README.md"; let path = cached_path(resource).await?; let meta = Meta::from_cache(&path).await?; assert_eq!(&meta.resource[..], resource);
pub fn is_fresh(&self) -> bool[src]
Check if resource is still fresh.