Struct cached_path::Options

source ·
pub struct Options {
    pub subdir: Option<String>,
    pub extract: bool,
}
Options to use with Cache::cached_path_with_options.

Fields

subdir: Option<String>

An optional subdirectory (relative to the cache root) to cache the resource in.

extract: bool

Automatically extract the resource, assuming the resource is an archive.

Implementations

Examples found in repository?
src/cache.rs (line 359)
354
355
356
357
358
359
360
361
    pub fn cached_path_in_subdir(
        &self,
        resource: &str,
        subdir: Option<&str>,
    ) -> Result<PathBuf, Error> {
        let options = Options::new(subdir, false);
        self.cached_path_with_options(resource, &options)
    }

The the cache subdirectory to use.

Treat the resource as an archive and try to extract it.