[−][src]Struct cached_path::Cache
When you need control over cache location or the HTTP client used to download
resources, you can create a Cache instance and then use the
instance method cached_path.
If you're going to be making more than a handful of calls to cached_path, this
is the recommended way to do it.
Fields
root: PathBufMethods
impl Cache[src]
pub async fn new() -> Result<Self, Error>[src]
Create a new Cache instance.
pub fn builder() -> CacheBuilder[src]
Create a CacheBuilder.
pub async fn cached_path<'_, '_>(
&'_ self,
resource: &'_ str
) -> Result<PathBuf, Error>[src]
&'_ self,
resource: &'_ str
) -> Result<PathBuf, Error>
Works just like cached_path.